import { defineStore } from "pinia";
import type {
  MaintenanceViewModel,
  CreateMaintenanceViewModel,
  UpdateMaintenanceViewModel,
} from "@/viewmodels/admin/unit/maintenance.models";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";

export const useMaintenanceStore = defineStore("maintenance", {
  state: () => {
    return {
      maintenances: [] as Array<MaintenanceViewModel & { tab_pos: number }>,
      totalCount: 0 as number,
      loading: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchMaintenances(offset = 0, count = 25, search = "", clear = false) {
      if (clear) this.maintenances = [];
      this.loading = "loading";
      //TODO enable fetch of done reports
      http
        .get(`/admin/maintenance?offset=${offset}&count=${count}${search != "" ? "&search=" + search : ""}`)
        .then((result) => {
          this.totalCount = result.data.total;
          result.data.maintenances
            .filter((elem: MaintenanceViewModel) => this.maintenances.findIndex((m) => m.id == elem.id) == -1)
            .map((elem: MaintenanceViewModel, index: number): MaintenanceViewModel & { tab_pos: number } => {
              return {
                ...elem,
                tab_pos: index + offset,
              };
            })
            .forEach((elem: MaintenanceViewModel & { tab_pos: number }) => {
              this.maintenances.push(elem);
            });
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    async getAllMaintenances(): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/maintenance?noLimit=true`).then((res) => {
        return { ...res, data: res.data.maintenances };
      });
    },
    async getMaintenancesByIds(ids: Array<string>): Promise<AxiosResponse<any, any>> {
      return await http
        .post(`/admin/maintenance/ids`, {
          ids,
        })
        .then((res) => {
          return { ...res, data: res.data.maintenances };
        });
    },
    async searchMaintenances(search: string): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/maintenance?search=${search}&noLimit=true`).then((res) => {
        return { ...res, data: res.data.maintenances };
      });
    },
    fetchMaintenanceById(id: string) {
      return http.get(`/admin/maintenance/${id}`);
    },
    async updateMaintenance(maintenance: UpdateMaintenanceViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.patch(`/admin/maintenance/${maintenance.id}`, {
        // TODO: data
      });
      this.fetchMaintenances();
      return result;
    },
  },
});
