import { defineStore } from "pinia";
import type {
  VehicleTypeViewModel,
  CreateVehicleTypeViewModel,
  UpdateVehicleTypeViewModel,
} from "@/viewmodels/admin/unit/vehicle/vehicleType.models";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";

export const useVehicleTypeStore = defineStore("vehicleType", {
  state: () => {
    return {
      vehicleTypes: [] as Array<VehicleTypeViewModel & { tab_pos: number }>,
      totalCount: 0 as number,
      loading: "loading" as "loading" | "fetched" | "failed",
      activeVehicleType: null as string | null,
      activeVehicleTypeObj: null as VehicleTypeViewModel | null,
      loadingActive: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchVehicleTypes(offset = 0, count = 25, search = "", clear = false) {
      if (clear) this.vehicleTypes = [];
      this.loading = "loading";
      http
        .get(`/admin/vehicleType?offset=${offset}&count=${count}${search != "" ? "&search=" + search : ""}`)
        .then((result) => {
          this.totalCount = result.data.total;
          result.data.vehicleTypes
            .filter((elem: VehicleTypeViewModel) => this.vehicleTypes.findIndex((m) => m.id == elem.id) == -1)
            .map((elem: VehicleTypeViewModel, index: number): VehicleTypeViewModel & { tab_pos: number } => {
              return {
                ...elem,
                tab_pos: index + offset,
              };
            })
            .forEach((elem: VehicleTypeViewModel & { tab_pos: number }) => {
              this.vehicleTypes.push(elem);
            });
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    async getAllVehicleTypes(): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/vehicleType?noLimit=true`).then((res) => {
        return { ...res, data: res.data.vehicleTypes };
      });
    },
    async searchVehicleTypes(search: string): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/vehicleType?search=${search}&noLimit=true`).then((res) => {
        return { ...res, data: res.data.vehicleTypes };
      });
    },
    fetchVehicleTypeByActiveId() {
      this.loadingActive = "loading";
      http
        .get(`/admin/vehicleType/${this.activeVehicleType}`)
        .then((res) => {
          this.activeVehicleTypeObj = res.data;
          this.loadingActive = "fetched";
        })
        .catch((err) => {
          this.loadingActive = "failed";
        });
    },
    fetchVehicleTypeById(id: string) {
      return http.get(`/admin/vehicleType/${id}`);
    },
    async createVehicleType(vehicleType: CreateVehicleTypeViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.post(`/admin/vehicleType`, {
        type: vehicleType.type,
        description: vehicleType.description,
      });
      this.fetchVehicleTypes();
      return result;
    },
    async updateActiveVehicleType(vehicleType: UpdateVehicleTypeViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.patch(`/admin/vehicleType/${vehicleType.id}`, {
        type: vehicleType.type,
        description: vehicleType.description,
      });
      this.fetchVehicleTypes();
      return result;
    },
    async deleteVehicleType(vehicleType: number): Promise<AxiosResponse<any, any>> {
      const result = await http.delete(`/admin/vehicleType/${vehicleType}`);
      this.fetchVehicleTypes();
      return result;
    },
  },
});
